#----------------------------------------------------
# (c) by Miguel A. Guerrero, 1998
# All rights reserved
#
# Creates an AutoCAD script that will create the 
# surface defined under the 'def_func' block. The
# current implementation evals the function defined
# in the file "surf_pol.ini"
#
# This file is customized for polar coordinates
#----------------------------------------------------

$pi=3.14159;

sub def_func {

	my ($s1,$s2) = @_;

	$r=eval($func);

	$x1= $r*cos($s2)*cos($s1);
	$x2= $r*cos($s2)*sin($s1);
	$f = $r*sin($s2);

	# Here we return the point result to main program
	($x1, $x2, $f);
}

#----------------------------------------------------
# Loads a configuration file
#----------------------------------------------------

sub load_ini {

   my ($file_name) = @_;
   my ($key, $contents);
   my (%ini) = ();

   open(fin, $file_name) || die "Can`t open $file_name\n";

   $key = "";
   $contents = "";

   while (<fin>) {

      chop;

      if (/\s*(.*)\s*\=\s*(.*)\s*/) {
         if (!($key eq "")) {
            $ini{$key} = $contents;
         }
         $key = $1;
         $contents = $2;
      }
      elsif (/^\#.*/) {
         # ignore input, is a commnent
      }
      elsif (/^\s*$/) {
         # ignore input, is an empty line
      }
      elsif (/^\.$/) {
         $contents .= "\n";
      }
      else {
         $contents .= "\n$_";
      }
   }
   close fin;

   $ini{$key} = $contents;
   %ini;
}

sub calc_expr {
	my ($s) = @_;
	$s=~ s/pi/\$pi/gi;
	eval($s);
}

#----------------------------------------------------
#  				Start of main program
#----------------------------------------------------

$file_cfg = "surf_pol.ini";

%cfg = &load_ini($file_cfg);

$func =  $cfg{def_func_r};
$func =~ s/pi/\$pi/gi;
$func =~ s/s1/\$s1/gi;
$func =~ s/s2/\$s2/gi;

$t1_ini = &calc_expr($cfg{s1_ini});
$t1_end = &calc_expr($cfg{s1_end});
$t1_pts = &calc_expr($cfg{s1_pts});

$t2_ini = &calc_expr($cfg{s2_ini});
$t2_end = &calc_expr($cfg{s2_end});
$t2_pts = &calc_expr($cfg{s2_pts});

# This creates a couple of points and sets the view
# point so that the screen shows how the surface is
# created in real time

($x1,$x2,$fx1x2) = &def_func ($t1_ini,$t2_ini);
printf ("_point %3.8f,%3.8f,%3.8f\n",$x1,$x2,$fx1x2);
($x1,$x2,$fx1x2) = &def_func ($t1_end,$t2_end);
printf ("_point %3.8f,%3.8f,%3.8f\n",$x1,$x2,$fx1x2);
print "_vpoint 1,.7,.7\n";
print "_zoom _extents\n";

print "_3dmesh\n";
print $t1_pts+1;
print "\n";
print $t2_pts+1;
print "\n";

for ($n1=0; $n1<=$t1_pts; $n1++) {

   $t1 = $t1_ini + ($t1_end - $t1_ini) / $t1_pts * $n1;

   for ($n2=0; $n2<=$t2_pts; $n2++) {

      $t2 = $t2_ini + ($t2_end - $t2_ini) / $t2_pts * $n2;

      ($x1,$x2,$fx1x2) = &def_func ($t1,$t2);

      printf ("%3.8f,%3.8f,%3.8f\n",$x1,$x2,$fx1x2);

  }
}

print "_vpoint 1,.7,.7\n";
print "_hide\n";

