#----------------------------------------------------
# (c) by Miguel A. Guerrero, 1998
# All rights reserved
#
# Creates an AutoCAD script that will create the 
# surface defined under the 'def_func' block. The
# current implementation evals the function defined
# in the file "surf_rec.ini"
#
# This file is customized for rectangular coordinates
#----------------------------------------------------

$pi=3.14159;

sub def_func {

   local ($t1,$t2) = @_;

   # s1 and s2 are the parameters that take values in the specified 
   # range. Here you could make a coordinate change if you wish 

   $x1 = eval($func_x);
   $x2 = eval($func_y);
   $f  = eval($func_z);

   # Here we return the point result to main program
   ($x1, $x2, $f);
}

#----------------------------------------------------
# Loads a configuration file
#----------------------------------------------------

sub load_ini {

   my ($file_name) = @_;
   my ($key, $contents);
   my (%ini) = ();

   open(fin, $file_name) || die "Can`t open $file_name\n";

   $key = "";
   $contents = "";

   while (<fin>) {

      chop;

      if (/\s*(.*)\s*\=\s*(.*)\s*/) {
         if (!($key eq "")) {
            $ini{$key} = $contents;
         }
         $key = $1;
         $contents = $2;
      }
      elsif (/^\#.*/) {
         # ignore input, is a commnent
      }
      elsif (/^\s*$/) {
         # ignore input, is an empty line
      }
      elsif (/^\.$/) {
         $contents .= "\n";
      }
      else {
         $contents .= "\n$_";
      }
   }
   close fin;

   $ini{$key} = $contents;
   %ini;
}

sub calc_expr {
	my ($s) = @_;
	$s=~ s/pi/\$pi/gi;
	eval($s);
}

#----------------------------------------------------
# 			Begin of main program
#----------------------------------------------------

$file_cfg = "surf_rec.ini";

%cfg = &load_ini($file_cfg);

$func_x =  $cfg{def_func_x};
$func_x =~ s/t1/\$t1/gi;
$func_x =~ s/t2/\$t2/gi;

$func_y =  $cfg{def_func_y};
$func_y =~ s/t1/\$t1/gi;
$func_y =~ s/t2/\$t2/gi;

$func_z =  $cfg{def_func_z};
$func_z =~ s/t1/\$t1/gi;
$func_z =~ s/t2/\$t2/gi;

$t1_ini = &calc_expr($cfg{t1_ini});
$t1_end = &calc_expr($cfg{t1_end});
$t1_pts = &calc_expr($cfg{t1_pts});

$t2_ini = &calc_expr($cfg{t2_ini});
$t2_end = &calc_expr($cfg{t2_end});
$t2_pts = &calc_expr($cfg{t2_pts});

# This creates a couple of points and sets the view
# point so that the screen shows how the surface is
# created in real time

($x1,$x2,$fx1x2) = &def_func ($t1_ini,$t2_ini);
printf ("_point %3.8f,%3.8f,%3.8f\n",$x1,$x2,$fx1x2);
($x1,$x2,$fx1x2) = &def_func ($t1_end,$t2_end);
printf ("_point %3.8f,%3.8f,%3.8f\n",$x1,$x2,$fx1x2);
print "_vpoint 1,.7,.7\n";
print "_zoom _extents\n";

print "_3dmesh\n";
print $t1_pts+1;
print "\n";
print $t2_pts+1;
print "\n";

for ($n1=0; $n1<=$t1_pts; $n1++) {

   $t1 = $t1_ini + ($t1_end - $t1_ini) / $t1_pts * $n1;

   for ($n2=0; $n2<=$t2_pts; $n2++) {

      $t2 = $t2_ini + ($t2_end - $t2_ini) / $t2_pts * $n2;

      ($x1,$x2,$fx1x2) = &def_func ($t1,$t2);

      printf ("%3.8f,%3.8f,%3.8f\n",$x1,$x2,$fx1x2);

   }
}

print "_vpoint 1,.7,.7\n";
print "_hide\n";

